@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2017-2021 - Swiss Data Science Center (SDSC)
+# Copyright 2017-2022 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -17,13 +17,13 @@
 # limitations under the License.
 """Renku exceptions."""
 
-import os
 from pathlib import Path
 from typing import List, Union
 
 import click
+from packaging.version import Version
 
-from renku.core.management import RENKU_HOME
+from renku.core.constant import RENKU_HOME
 
 
 class RenkuException(Exception):
@@ -33,6 +33,22 @@ class RenkuException(Exception):
     """
 
 
+class DatasetException(RenkuException):
+    """Base class for all dataset-related exceptions."""
+
+
+class ActivityDownstreamNotEmptyError(RenkuException):
+    """Raised when an activity cannot be deleted because its downstream is not empty."""
+
+    def __init__(self, activity):
+        self.activity = activity
+        super().__init__(f"Activity '{activity.id}' has non-empty downstream")
+
+
+class LockError(RenkuException):
+    """Raise when a project cannot be locked."""
+
+
 class RequestError(RenkuException):
     """Raise when a ``requests`` call fails."""
 
@@ -56,6 +72,15 @@ class ParameterError(RenkuException):
         super().__init__(message)
 
 
+class IncompatibleParametersError(ParameterError):
+    """Raise in case of incompatible parameters/flags."""
+
+    def __init__(self, a: str = None, b: str = None):
+        """Build a custom message."""
+        message = f"{a} is incompatible with {b}" if a is not None and b is not None else "Incompatible parameters"
+        super().__init__(message)
+
+
 class InvalidFileOperation(RenkuException):
     """Raise when trying to perform invalid file operation."""
 
@@ -148,14 +173,6 @@ class NothingToCommit(RenkuException):
         super(NothingToCommit, self).__init__("There is nothing to commit.")
 
 
-class DatasetFileExists(RenkuException):
-    """Raise when file is already in dataset."""
-
-    def __init__(self):
-        """Build a custom message."""
-        super(DatasetFileExists, self).__init__("File already exists in dataset. Use --force to add.")
-
-
 class CommitMessageEmpty(RenkuException):
     """Raise invalid commit message."""
 
@@ -202,28 +219,16 @@ class InvalidOutputPath(RenkuException):
 class OutputsNotFound(RenkuException):
     """Raise when there are not any detected outputs in the repository."""
 
-    def __init__(self, repository, inputs):
+    def __init__(self):
         """Build a custom message."""
-        from pathlib import Path
 
-        msg = "There are not any detected outputs in the repository."
-
-        paths = [
-            os.path.relpath(input_.default_value)  # relative to cur path
-            for input_ in inputs
-            if Path(input_.default_value).is_dir()
-        ]
-
-        if paths:
-            msg += (
-                '\n  (use "git rm <file>..." to remove them first)'
-                "\n\n" + "\n".join("\t" + click.style(path, fg="yellow") for path in paths) + "\n\n"
-                "Once you have removed files that should be used as outputs,\n"
-                "you can safely rerun the previous command."
-                "\nYou can use --output flag to specify outputs explicitly."
-            )
-        else:
-            msg += "\n\nIf you want to track the command anyway use " "--no-output option."
+        msg = (
+            "There are not any detected outputs in the repository. This can be due to your command not creating "
+            "any new files or due to files that get created already existing before the command was run. In the "
+            "latter case, you can remove those files prior to running your command.\nIf you want to track the command"
+            "without outputs, use the use --no-output option.\nYou can also use the --output flag to track outputs"
+            "manually."
+        )
 
         super(OutputsNotFound, self).__init__(msg)
 
@@ -246,7 +251,7 @@ class InvalidSuccessCode(RenkuException):
         super(InvalidSuccessCode, self).__init__(msg)
 
 
-class DatasetNotFound(RenkuException):
+class DatasetNotFound(DatasetException):
     """Raise when dataset is not found."""
 
     def __init__(self, *, name=None, message=None):
@@ -254,13 +259,52 @@ class DatasetNotFound(RenkuException):
         if message:
             msg = message
         elif name:
-            msg = f'Dataset "{name}" is not found.'
+            msg = f"Dataset '{name}' is not found."
         else:
             msg = "Dataset is not found."
         super().__init__(msg)
 
 
-class DatasetExistsError(RenkuException):
+class DatasetTagNotFound(DatasetException):
+    """Raise when a tag can't be found."""
+
+    def __init__(self, tag) -> None:
+        msg = f"Couldn't find dataset tag '{tag}'."
+        super().__init__(msg)
+
+
+class FileNotFound(RenkuException):
+    """Raise when a file is not found."""
+
+    def __init__(self, path, checksum=None, revision=None):
+        """Build a custom message."""
+        if checksum:
+            message = f"File not found in the repository: {checksum}:{path}"
+        elif revision:
+            message = f"File not found in the repository: {path}@{revision}"
+        else:
+            message = f"Cannot find file {path}"
+
+        super().__init__(message)
+
+
+class ExternalFileNotFound(DatasetException):
+    """Raise when an external file is not found."""
+
+    def __init__(self, path):
+        """Build a custom message."""
+        super().__init__(f"Cannot find external file '{path}'")
+
+
+class DirectoryNotEmptyError(RenkuException):
+    """Raised when a directory passed as output is not empty."""
+
+    def __init__(self, path):
+        """Build a custom message."""
+        super().__init__(f"Destination directory is not empty: '{path}'")
+
+
+class DatasetExistsError(DatasetException):
     """Raise when trying to create an existing dataset."""
 
 
@@ -388,10 +432,6 @@ class GitLFSError(RenkuException):
     """Raised when a Git LFS operation fails."""
 
 
-class UrlSchemeNotSupported(RenkuException):
-    """Raised when adding data from unsupported URL schemes."""
-
-
 class OperationError(RenkuException):
     """Raised when an operation at runtime raises an error."""
 
@@ -429,7 +469,7 @@ class WorkflowRerunError(RenkuException):
         super(WorkflowRerunError, self).__init__(msg)
 
 
-class ExportError(RenkuException):
+class ExportError(DatasetException):
     """Raised when a dataset cannot be exported."""
 
 
@@ -441,6 +481,10 @@ class InvalidTemplateError(TemplateError):
     """Raised when using a non-valid template."""
 
 
+class TemplateMissingReferenceError(TemplateError):
+    """Raised when using a non-valid template."""
+
+
 class TemplateUpdateError(TemplateError):
     """Raised when a project couldn't be updated from its template."""
 
@@ -457,14 +501,9 @@ class MigrationError(RenkuException):
     """Raised when something went wrong during migrations."""
 
 
-class RenkuImportError(RenkuException):
+class ImportError(RenkuException):
     """Raised when a dataset cannot be imported."""
 
-    def __init__(self, exp, msg):
-        """Embed exception and build a custom message."""
-        self.exp = exp
-        super(RenkuImportError, self).__init__(msg)
-
 
 class CommandNotFinalizedError(RenkuException):
     """Raised when a non-finalized command is executed."""
@@ -478,7 +517,7 @@ class RenkuSaveError(RenkuException):
     """Raised when renku save doesn't work."""
 
 
-class DatasetImageError(RenkuException):
+class DatasetImageError(DatasetException):
     """Raised when a local dataset image is not accessible."""
 
 
@@ -551,9 +590,9 @@ class GraphCycleError(RenkuException):
 
     def __init__(self, cycles: List[List[str]]):
         """Embed exception and build a custom message."""
-        cycles = "), (".join(", ".join(cycle) for cycle in cycles)
+        cycle_str = "), (".join(", ".join(cycle) for cycle in cycles)
         super().__init__(
-            f"Cycles detected in execution graph: ({cycles})\nCircular workflows are not supported in renku\n"
+            f"Cycles detected in execution graph: ({cycle_str})\nCircular workflows are not supported in renku\n"
             "If this happened as part of a 'renku run' or 'renku workflow execute', please git reset and clean"
             "the project and try again. This might be due to renku erroneously detecting an input as an output, "
             "if so, please specify the wrongly detected output as an explicit input using '--input'."
@@ -566,3 +605,66 @@ class NothingToExecuteError(RenkuException):
 
 class TerminalSizeError(RenkuException):
     """Raised when terminal is too small for a command."""
+
+
+class DockerError(RenkuException):
+    """Raised when error has occurred while executing docker command."""
+
+    def __init__(self, reason: str):
+        """Embed exception and build a custom message."""
+        super().__init__(f"Docker failed: {reason}")
+
+
+class RenkulabSessionError(RenkuException):
+    """Raised when an error occurs trying to start sessions with the notebook service."""
+
+
+class RenkulabSessionGetUrlError(RenkuException):
+    """Raised when Renku deployment's URL cannot be gotten from project's remotes or configured remotes."""
+
+    def __init__(self):
+        message = (
+            "Cannot determine the Renku deployment's URL. Ensure your current project is a valid Renku project and has "
+            "a remote URL."
+        )
+        super().__init__(message)
+
+
+class NotebookSessionNotReadyError(RenkuException):
+    """Raised when a user attempts to open a session that is not ready."""
+
+
+class NotebookSessionImageNotExistError(RenkuException):
+    """Raised when a user attempts to start a session with an image that does not exist."""
+
+
+class MetadataMergeError(RenkuException):
+    """Raise when merging of metadata failed."""
+
+
+class MinimumVersionError(RenkuException):
+    """Raised when accessing a project whose minimum version is larger than the current renku version."""
+
+    def __init__(self, current_version: Version, minimum_version: Version) -> None:
+        self.current_version = current_version
+        self.minimum_version = minimum_version
+
+        super().__init__(
+            f"You are using renku version {current_version} but this project requires at least version "
+            f"{minimum_version}. Please upgrade renku to work on this project."
+        )
+
+
+class DatasetProviderNotFound(DatasetException, ParameterError):
+    """Raised when a dataset provider cannot be found based on a URL or a provider name."""
+
+    def __init__(self, *, name: str = None, uri: str = None, message: str = None):
+        if message is None:
+            if name:
+                message = f"Provider '{name}' not found"
+            elif uri:
+                message = f"Cannot find a provider to process '{uri}'"
+            else:
+                message = "Provider not found"
+
+        super().__init__(message)
