@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2017-2021 - Swiss Data Science Center (SDSC)
+# Copyright 2017-2022 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -16,6 +16,3 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Renku repository management."""
-
-RENKU_HOME = ".renku"
-"""Project directory name."""
