@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2018-2021 - Swiss Data Science Center (SDSC)
+# Copyright 2018-2022 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -27,28 +27,43 @@ from collections import defaultdict
 from pathlib import Path
 from shutil import move, which
 from subprocess import PIPE, STDOUT, check_output, run
-from typing import Optional
+from typing import TYPE_CHECKING, List, Optional, Tuple
 
 import attr
 import pathspec
 from werkzeug.utils import cached_property
 
 from renku.core import errors
-from renku.core.models.entity import Entity
-from renku.core.models.provenance.activity import Collection
-from renku.core.utils import communication
-from renku.core.utils.file_size import parse_file_size
-from renku.core.utils.git import run_command
+from renku.core.management.git import _expand_directories
+from renku.core.management.repository import RepositoryApiMixin  # type: ignore
+from renku.core.util import communication
+from renku.core.util.file_size import parse_file_size
+from renku.core.util.git import run_command
 
-from .git import _expand_directories
-from .repository import RepositoryApiMixin
+if TYPE_CHECKING:
+    from renku.domain_model.entity import Entity  # type: ignore
+
+
+class RenkuGitWildMatchPattern(pathspec.patterns.GitWildMatchPattern):
+    """Custom GitWildMatchPattern matcher."""
+
+    __slots__ = ("pattern",)
+
+    def __init__(self, pattern, include=None):
+        """Initialize RenkuRegexPattern."""
+        super().__init__(pattern, include)
+        self.pattern = pattern
+
+
+pathspec.util.register_pattern("renku_gitwildmatch", RenkuGitWildMatchPattern)
 
 
 def check_external_storage_wrapper(fn):
     """Check availability of external storage on methods that need it.
 
-    :raises: ``errors.ExternalStorageNotInstalled``
-    :raises: ``errors.ExternalStorageDisabled``
+    Raises:
+        ``errors.ExternalStorageNotInstalled``: If external storage isn't installed.
+        ``errors.ExternalStorageDisabled``: If external storage isn't enabled.
     """
     # noqa
     @functools.wraps(fn)
@@ -107,8 +122,9 @@ class StorageApiMixin(RepositoryApiMixin):
     def check_external_storage(self):
         """Check if repository has external storage enabled.
 
-        :raises: ``errors.ExternalStorageNotInstalled``
-        :raises: ``errors.ExternalStorageDisabled``
+        Raises:
+            ``errors.ExternalStorageNotInstalled``: If external storage isn't installed.
+            ``errors.ExternalStorageDisabled``: If external storage isn't enabled.
         """
         storage_installed = self.storage_installed_locally and self.storage_installed
         if self.external_storage_requested and not storage_installed:
@@ -123,13 +139,12 @@ class StorageApiMixin(RepositoryApiMixin):
     def renku_lfs_ignore(self):
         """Gets pathspec for files to not add to LFS."""
         ignore_path = self.path / self.RENKU_LFS_IGNORE_PATH
-        renku_protected_paths = ["/.renku"]
 
         if not os.path.exists(ignore_path):
-            return pathspec.PathSpec.from_lines("renku_gitwildmatch", renku_protected_paths)
+            return pathspec.PathSpec.from_lines("renku_gitwildmatch", self.RENKU_PROTECTED_PATHS)
         with ignore_path.open("r") as f:
             # NOTE: Append `renku_protected_paths` at the end to give it the highest priority
-            lines = itertools.chain(f, renku_protected_paths)
+            lines = itertools.chain(f, self.RENKU_PROTECTED_PATHS)
             return pathspec.PathSpec.from_lines("renku_gitwildmatch", lines)
 
     @property
@@ -418,8 +433,18 @@ class StorageApiMixin(RepositoryApiMixin):
 
         return track_paths
 
-    def get_lfs_migrate_filters(self):
+    def get_lfs_migrate_filters(self) -> Tuple[List[str], List[str]]:
         """Gets include, exclude and above filters for lfs migrate."""
+
+        def add_migrate_pattern(pattern, collection):
+            if pattern in self.RENKU_PROTECTED_PATHS:
+                return
+            pattern = pattern.strip()
+            if pattern.endswith("*"):
+                return
+            pattern = pattern.rstrip("/")
+            collection.append(f"{pattern}/**")
+
         includes = []
         excludes = []
         for p in self.renku_lfs_ignore.patterns:
@@ -432,27 +457,32 @@ class StorageApiMixin(RepositoryApiMixin):
 
             if p.include:  # File ignored by LFS
                 excludes.append(pattern)
+                add_migrate_pattern(pattern, excludes)
             else:
                 includes.append(pattern)
+                add_migrate_pattern(pattern, includes)
 
         if excludes:
             excludes = ["--exclude", ",".join(excludes)]
         if includes:
             includes = ["--include", ",".join(includes)]
 
-        above = ["--above", str(self.minimum_lfs_file_size)]
+        return includes, excludes
 
-        return includes, excludes, above
-
-    def check_lfs_migrate_info(self, everything=False):
+    def check_lfs_migrate_info(self, everything=False, use_size_filter=True):
         """Return list of file groups in history should be in LFS."""
         ref = ["--everything"] if everything else ["--include-ref", self.repository.active_branch.name]
 
-        includes, excludes, above = self.get_lfs_migrate_filters()
+        includes, excludes = self.get_lfs_migrate_filters()
 
         ignore_pointers = ["--pointers", "ignore"]
 
-        command = self._CMD_STORAGE_MIGRATE_INFO + ref + above + includes + excludes
+        command = self._CMD_STORAGE_MIGRATE_INFO + ref + includes + excludes
+
+        # NOTE: ``lfs migrate info`` supports ``--above`` while ``lfs migrate import`` doesn't.
+        if use_size_filter:
+            above = ["--above", str(self.minimum_lfs_file_size)]
+            command += above
 
         try:
             lfs_output = run(
@@ -479,28 +509,26 @@ class StorageApiMixin(RepositoryApiMixin):
             if match:
                 groups.append(match.groups()[0])
 
+        if groups and use_size_filter:
+            # NOTE: Since there are some large files, remove the size filter so that users get list of all files that
+            # will be moved to LFS.
+            return self.check_lfs_migrate_info(everything=everything, use_size_filter=False)
+
         return groups
 
     def migrate_files_to_lfs(self, paths):
         """Migrate files to Git LFS."""
-        if not self.has_graph_files:
-            raise errors.OperationError(
-                "This command is only supported with the new graph metadata, which doesn't exist. "
-                "Create it by running `renku graph generate`."
-            )
-
         if paths:
             includes = ["--include", ",".join(paths)]
             excludes = []
-            above = []
         else:
-            includes, excludes, above = self.get_lfs_migrate_filters()
+            includes, excludes = self.get_lfs_migrate_filters()
 
         tempdir = Path(tempfile.mkdtemp())
         map_path = tempdir / "objectmap.csv"
         object_map = [f"--object-map={map_path}"]
 
-        command = self._CMD_STORAGE_MIGRATE_IMPORT + above + includes + excludes + object_map
+        command = self._CMD_STORAGE_MIGRATE_IMPORT + includes + excludes + object_map
 
         try:
             lfs_output = run(command, stdout=PIPE, stderr=STDOUT, cwd=self.path, universal_newlines=True)
@@ -543,10 +571,13 @@ class StorageApiMixin(RepositoryApiMixin):
                     processed.add(path_obj)
                     path_obj = path_obj.parent
 
-        def _map_checksum(entity, checksum_mapping) -> Optional[Entity]:
+        def _map_checksum(entity, checksum_mapping) -> Optional["Entity"]:
             """Update the checksum and id of an entity based on a mapping."""
+            from renku.domain_model.entity import Entity
+            from renku.domain_model.provenance.activity import Collection
+
             if entity.checksum not in checksum_mapping:
-                return
+                return None
 
             new_checksum = checksum_mapping[entity.checksum]
 
@@ -555,10 +586,10 @@ class StorageApiMixin(RepositoryApiMixin):
                 for member in entity.members:
                     new_member = _map_checksum(member, checksum_mapping)
                     if new_member:
-                        member.append(new_member)
+                        members.append(new_member)
                     else:
                         members.append(member)
-                new_entity = Collection(checksum=new_checksum, path=entity.path, members=members)
+                new_entity: Entity = Collection(checksum=new_checksum, path=entity.path, members=members)
             else:
                 new_entity = Entity(checksum=new_checksum, path=entity.path)
 
@@ -567,7 +598,7 @@ class StorageApiMixin(RepositoryApiMixin):
         def _map_checksum_old(entity, checksum_mapping):
             """Update the checksum and id of an entity based on a mapping."""
             # TODO: Remove this method once moved to Entity with 'id' field
-            from renku.core.models.provenance.activity import Collection
+            from renku.domain_model.provenance.activity import Collection
 
             if entity.checksum not in checksum_mapping:
                 return
