@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2017-%d - Swiss Data Science Center (SDSC)
+# Copyright 2017-2022 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -22,10 +22,11 @@ import re
 try:
     from importlib.metadata import distribution
 except ImportError:
-    from importlib_metadata import distribution
+    from importlib_metadata import distribution  # type: ignore
 
-__version__ = "1.1.4"
+__version__ = "1.6.0"
 __template_version__ = "0.3.1"
+__minimum_project_version__ = "1.2.0"
 
 
 def is_release():
@@ -40,7 +41,7 @@ def _get_distribution_url():
         d = distribution("renku")
         return d.metadata["Home-page"]
     except Exception:
-        return "N/A"
+        return "https://github.com/swissdatasciencecenter/renku-python"
 
 
-version_url = "{}/tree/{}".format(_get_distribution_url(), "v" + __version__)
+version_url = f"{_get_distribution_url()}/tree/v{__version__}"
